@@ -26,6 +26,7 @@
 #include <string.h>
 #include <unistd.h>
 #include "manette-event-private.h"
+#include "manette-mapping-manager.h"
 
 struct _ManetteDevice
 {
@@ -754,7 +755,15 @@ manette_device_new (const gchar  *filename,
   return self;
 }
 
-// FIXME move to ManetteMappingManager?
+/**
+ * manette_device_get_guid:
+ * @self: a #ManetteDevice
+ *
+ * Gets the identifier used by SDL mappings to discriminate game controller
+ * devices.
+ *
+ * Returns: (transfer none): the identifier used by SDL mappings
+ */
 const gchar *
 manette_device_get_guid (ManetteDevice *self)
 {
@@ -766,6 +775,30 @@ manette_device_get_guid (ManetteDevice *self)
   return self->guid;
 }
 
+/**
+ * manette_device_has_input:
+ * @self: a #ManetteDevice
+ * @type: the input type
+ * @code: the input code
+ *
+ * Gets whether the device has the given input. If the input is present it means
+ * that the device can send events for it regardless of whether the device is
+ * mapped or not.
+ *
+ * Returns: whether the device has the given input
+ */
+gboolean
+manette_device_has_input (ManetteDevice *self,
+                          guint          type,
+                          guint          code)
+{
+  g_return_val_if_fail (MANETTE_IS_DEVICE (self), FALSE);
+
+  return MANETTE_IS_MAPPING (self->mapping) ?
+    manette_mapping_has_destination_input (self->mapping, type, code) :
+    libevdev_has_event_code (self->evdev_device, type, code);
+}
+
 /**
  * manette_device_get_name:
  * @self: a #ManetteDevice
@@ -793,6 +826,8 @@ manette_device_get_name (ManetteDevice *self)
 int
 manette_device_get_product_id (ManetteDevice *self)
 {
+  g_return_val_if_fail (MANETTE_IS_DEVICE (self), 0);
+
   return libevdev_get_id_product (self->evdev_device);
 }
 
@@ -807,6 +842,8 @@ manette_device_get_product_id (ManetteDevice *self)
 int
 manette_device_get_vendor_id (ManetteDevice *self)
 {
+  g_return_val_if_fail (MANETTE_IS_DEVICE (self), 0);
+
   return libevdev_get_id_vendor (self->evdev_device);
 }
 
@@ -821,6 +858,8 @@ manette_device_get_vendor_id (ManetteDevice *self)
 int
 manette_device_get_bustype_id (ManetteDevice *self)
 {
+  g_return_val_if_fail (MANETTE_IS_DEVICE (self), 0);
+
   return libevdev_get_id_bustype (self->evdev_device);
 }
 
@@ -835,6 +874,8 @@ manette_device_get_bustype_id (ManetteDevice *self)
 int
 manette_device_get_version_id (ManetteDevice *self)
 {
+  g_return_val_if_fail (MANETTE_IS_DEVICE (self), 0);
+
   return libevdev_get_id_version (self->evdev_device);
 }
 
@@ -843,8 +884,83 @@ void
 manette_device_set_mapping (ManetteDevice  *self,
                             ManetteMapping *mapping)
 {
+  g_return_if_fail (MANETTE_IS_DEVICE (self));
+
   if (self->mapping != NULL)
     g_object_unref (self->mapping);
 
   self->mapping = mapping ? g_object_ref (mapping) : NULL;
 }
+
+/**
+ * manette_device_has_user_mapping:
+ * @self: a #ManetteDevice
+ *
+ * Gets whether @self has a user mapping.
+ *
+ * Returns: whether @self has a user mapping
+ */
+gboolean
+manette_device_has_user_mapping (ManetteDevice *self)
+{
+  const gchar *guid;
+  ManetteMappingManager *mapping_manager;
+  gboolean has_user_mapping;
+
+  g_return_val_if_fail (MANETTE_IS_DEVICE (self), FALSE);
+
+  guid = manette_device_get_guid (self);
+  mapping_manager = manette_mapping_manager_new ();
+  has_user_mapping = manette_mapping_manager_has_user_mapping (mapping_manager, guid);
+  g_object_unref (mapping_manager);
+
+  return has_user_mapping;
+}
+
+/**
+ * manette_device_save_user_mapping:
+ * @self: a #ManetteDevice
+ * @mapping_string: the mapping string
+ *
+ * Saves @mapping_string as the user mapping for @self.
+ */
+void
+manette_device_save_user_mapping (ManetteDevice *self,
+                                  const gchar   *mapping_string)
+{
+  const gchar *guid;
+  const gchar *name;
+  ManetteMappingManager *mapping_manager;
+
+  g_return_if_fail (MANETTE_IS_DEVICE (self));
+  g_return_if_fail (mapping_string != NULL);
+
+  guid = manette_device_get_guid (self);
+  name = manette_device_get_name (self);
+  mapping_manager = manette_mapping_manager_new ();
+  manette_mapping_manager_save_mapping (mapping_manager,
+                                        guid,
+                                        name,
+                                        mapping_string);
+  g_object_unref (mapping_manager);
+}
+
+/**
+ * manette_device_remove_user_mapping:
+ * @self: a #ManetteDevice
+ *
+ * Removes the user mapping for @self.
+ */
+void
+manette_device_remove_user_mapping (ManetteDevice *self)
+{
+  const gchar *guid;
+  ManetteMappingManager *mapping_manager;
+
+  g_return_if_fail (MANETTE_IS_DEVICE (self));
+
+  guid = manette_device_get_guid (self);
+  mapping_manager = manette_mapping_manager_new ();
+  manette_mapping_manager_delete_mapping (mapping_manager, guid);
+  g_object_unref (mapping_manager);
+}
