@@ -31,7 +31,14 @@ G_BEGIN_DECLS
 
 G_DECLARE_FINAL_TYPE (ManetteDevice, manette_device, MANETTE, DEVICE, GObject)
 
+gboolean manette_device_has_input (ManetteDevice *self,
+                                   guint          type,
+                                   guint          code);
 const gchar *manette_device_get_name (ManetteDevice *self);
+gboolean manette_device_has_user_mapping (ManetteDevice *self);
+void manette_device_save_user_mapping (ManetteDevice *self,
+                                       const gchar   *mapping_string);
+void manette_device_remove_user_mapping (ManetteDevice *self);
 
 G_END_DECLS
 
